@@ -1,5 +1,5 @@
 # $File: //member/autrijus/cpanplus/dist/lib/CPANPLUS/Internals/Make.pm $
-# $Revision: #12 $ $Change: 3846 $ $DateTime: 2002/04/10 05:54:28 $
+# $Revision: #19 $ $Change: 4057 $ $DateTime: 2002/04/30 15:37:33 $
 
 #######################################################
 ###             CPANPLUS/Internals/Make.pm          ###
@@ -36,7 +36,9 @@ sub _make {
     my $data   = $args{'module'};
 
     ### make target: may also be 'makefile', 'make', 'test' or 'skiptest'
-    my $target = lc($args{'target'}) || 'install';
+    my $target        = lc($args{'target'}) || 'install';
+    ### target of prereqs: may also be 'makefile', 'make', 'test' or 'skiptest'
+    my $prereq_target = lc($args{'prereq_target'}) || 'install';
 
     ### test for Backend->flush
     ###print "this is todo: ", Dumper $self->{_todo};
@@ -61,7 +63,7 @@ sub _make {
     $report    = $conf->get_conf('cpantest')       unless defined $report;
     $mmflags   = $conf->get_conf('makemakerflags') unless defined $mmflags;
     $makeflags = $conf->get_conf('makeflags')      unless defined $makeflags;
-
+ 
     my $verbose = $conf->get_conf('verbose');
     my $captured; # capture buffer of _run
 
@@ -106,8 +108,10 @@ sub _make {
 
         my @args = @{$self->_flags_arrayref($mmflags)};
 
+        ### BIG CAVEAT: We're forced to use the $|=1 trick to ensure proper
+        ###             ordering of STDIN, STDOUT and STDERR in captured buffer
         unless( $self->_run(
-            command => [$perl, 'Makefile.PL', @args],
+            command => [$perl, '-e', '$|=1;do"Makefile.PL"', @args],
             buffer  => \$captured,
             verbose => 1
         ) ) {
@@ -149,12 +153,6 @@ sub _make {
                 );
 
                 $flag = 1;
-
-            } elsif ( defined $self->{_todo}->{failed}->{$mod} ) {
-                $err->inform(
-                    msg => "According to the cache, prerequisite $mod is already installed",
-                    quiet => !$verbose
-                );
             }
         }
 
@@ -212,6 +210,18 @@ sub _make {
                 next;
             }
 
+            if ( grep { $_->{module}{module} eq $mod } @{$self->{_todo}{make}} ) {
+                $err->trap( error => "Recursive dependency detected in $mod, skipping" );
+                next;
+            }
+            elsif ( defined $self->{_todo}->{failed}->{$mod} ) {
+                $err->inform(
+                    msg => "According to the cache, prerequisite $mod is already installed",
+                    quiet => !$verbose
+                );
+                next;
+            }
+
             ### check if we're in shell mode, and if we should ask to follow prereqs.
             ### should probably use words rather than numbers -kane
             if ( $self->{_shell} and $conf->get_conf('prereqs') == 2 ) {
@@ -244,7 +254,11 @@ sub _make {
         } keys %list ];
 
         while (my $mod_ref = shift @{$self->{_todo}->{install}} ) {
-            $self->_install_module( modules => $mod_ref );
+            $self->_install_module(
+                modules       => $mod_ref,
+                target        => $prereq_target,
+                prereq_target => $prereq_target,
+            );
         }
 
     } else {
@@ -282,7 +296,7 @@ sub _make {
 
             if ($target ne 'skiptest') {
                 unless ( $self->_run(
-                    command => [$make, @args, 'test'],
+                    command => [$make, @args, 'test', "TEST_VERBOSE=(eval(chr(36).'|=1'),0)"],
                     buffer  => \$captured,
                     verbose => 1
                 ) ) {
@@ -291,11 +305,16 @@ sub _make {
 
                     $err->trap( error => "MAKE TEST failed! - $!" );
                     $self->_send_report( module => $data, buffer => $captured) if $report;
-                    $self->_restore_startdir;
-                    return 0;
+
+                    unless ($force) {
+                        $self->_restore_startdir;
+                        return 0;
+                    }
+                }
+                else {
+                    $self->_send_report( module => $data, buffer => $captured) if $report;
                 }
 
-                $self->_send_report( module => $data, buffer => $captured) if $report;
                 last INSTALL if $target eq 'test';
             }
 
@@ -324,7 +343,8 @@ sub _make {
 
     ### if we still have modules left to do in our _tomake list, this is the time to do it!
     if ( @{$self->{_todo}->{make}} ) {
-        $self->_make( %{ shift @{$self->{_todo}->{make}} } );
+        $self->_make( %{ $self->{_todo}{make}->[0] } );
+        shift @{$self->{_todo}{make}};
     }
 
     ### indicate success ###
@@ -456,168 +476,11 @@ sub _find_prereq {
     return \%p;
 }
 
-
-### Execute a command: $cmd may be a scalar or an arrayref of cmd and args
-### $bufout is an scalar ref to store outputs, $verbose can override conf
-sub _run {
-    my ($self, %args) = @_;
-    my ($cmd, $buffer, $verbose) = @args{qw|command buffer verbose|};
-    my $err = $self->{_error};
-    my ($buferr, $bufout);
-
-    $$buffer = '';
-    $verbose = $self->{_conf}->get_conf('verbose')
-        unless defined $verbose;
-
-    ### STDOUT message handler
-    my $_out_handler = sub {
-        my $buf = shift; print STDOUT $buf if $verbose;
-        $$buffer .= $buf; $bufout .= $buf;
-        $err->inform( msg => $1, quiet => 1 ) while $bufout =~ s/(.*)\n//;
-    };
-
-    ### STDERR message handler
-    my $_err_handler = sub {
-        my $buf = shift; print STDERR $buf if $verbose;
-        $$buffer .= $buf; $buferr .= $buf;
-        $err->trap( error => $1, quiet => 1 ) while $buferr =~ s/(.*)\n//;
-    };
-
-    my @cmd = ref($cmd) ? grep(length, @{$cmd}) : $cmd;
-    my $is_win98 = ($^O eq 'MSWin32' and !Win32::IsWinNT());
-
-    ### inform the user. note that we don't want to used mangled $verbose.
-    $err->inform(
-        msg   => "Running [@cmd]...",
-        quiet => !$self->{_conf}->get_conf('verbose'),
-    );
-
-    ### First, we prefer Barrie Slaymaker's wonderful IPC::Run module.
-    if (!$is_win98 and $self->_can_use(
-        modules  => { 'IPC::Run' => '0.55' },
-        complain => ($^O eq 'MSWin32'),
-    ) ) {
-        STDOUT->autoflush(1); STDERR->autoflush(1); 
-        IPC::Run::run(\@cmd, \*STDIN, $_out_handler, $_err_handler);
-    }
-
-    ### Next, IPC::Open3 is know to fail on Win32, but works on Un*x.
-    elsif ($^O ne 'MSWin32' and $self->_can_use(
-        modules => { map { $_ => '0.0' } qw|IPC::Open3 IO::Select Symbol| },
-    ) ) {
-        $self->_open3_run(\@cmd, $_out_handler, $_err_handler);
-    }
-
-    ### Abandon all hope; falls back to simple system() on verbose calls.
-    elsif ($verbose) {
-        system(@cmd);
-    }
-
-    ### Non-verbose system() needs to have STDOUT and STDERR muted.
-    else {
-        local *SAVEOUT; local *SAVEERR;
-
-        open(SAVEOUT, ">&STDOUT")
-            or ($err->trap( error => "couldn't dup STDOUT: $!" ), return);
-        open(STDOUT, ">".File::Spec->devnull)
-            or ($err->trap( error => "couldn't reopen STDOUT: $!" ), return);
-
-        open(SAVEERR, ">&STDERR")
-            or ($err->trap( error => "couldn't dup STDERR: $!" ), return);
-        open(STDERR, ">".File::Spec->devnull)
-            or ($err->trap( error => "couldn't reopen STDERR: $!" ), return);
-
-        system(@cmd);
-
-        open(STDOUT, ">&SAVEOUT")
-            or ($err->trap( error => "couldn't restore STDOUT: $!" ), return);
-        open(STDERR, ">&SAVEERR")
-            or ($err->trap( error => "couldn't restore STDERR: $!" ), return);
-    }
-
-    $_out_handler->("\n") if length $bufout;
-    $_err_handler->("\n") if length $buferr;
-
-    return !$?;
-}
-
-
-### IPC::Run::run emulator, using IPC::Open3.
-sub _open3_run {
-    my ($self, $cmdref, $_out_handler, $_err_handler) = @_;
-    my $err = $self->{_error};
-    my @cmd = @{$cmdref};
-
-    ### Following code are adapted from Friar 'abstracts' in the
-    ### Perl Monastery (http://www.perlmonks.org/index.pl?node_id=151886).
-
-    local *SAVEIN;
-
-    ### saving away STDIN first, because it will be closed by open3
-    unless (open(SAVEIN, ">&STDIN")) {
-        $err->trap( error => "couldn't dup STDIN: $!" );
-        return 0;
-    }
-
-    my ($outfh, $errfh); # open3 handles
-
-    my $pid = eval {
-        IPC::Open3::open3(
-            '<&STDIN', # may also be \*STDIN according to Barrie
-            $outfh = Symbol::gensym(),
-            $errfh = Symbol::gensym(),
-            @cmd,
-        )
-    };
-
-    if ($@) {
-        $err->trap( error => "couldn't spawn process: $@" );
-        return;
-    }
-
-    my $sel = IO::Select->new; # create a select object
-    $sel->add($outfh, $errfh); # and add the fhs
-
-    STDOUT->autoflush(1); STDERR->autoflush(1);
-    $outfh->autoflush(1) if UNIVERSAL::can($outfh, 'autoflush');
-    $errfh->autoflush(1) if UNIVERSAL::can($errfh, 'autoflush');
-
-    while (my @ready = $sel->can_read) {
-        foreach my $fh (@ready) { # loop through buffered handles
-            # read up to 4096 bytes from this fh.
-            my $len = sysread $fh, my($buf), 4096;
-
-            if (not defined $len){
-                # There was an error reading
-                $err->trap( error => "Error from child: $!" );
-                return;
-            }
-            elsif ($len == 0){
-                $sel->remove($fh); # finished reading
-                next;
-            }
-            elsif ($fh == $outfh) {
-                $_out_handler->($buf);
-            } elsif ($fh == $errfh) {
-                $_err_handler->($buf);
-            } else {
-                $err->trap( error => "IO::Select error" );
-                return;
-            }
-        }
-    }
-
-    waitpid $pid, 0; # wait for it to die
-
-    ### restore STDIN
-    local $^W; # quell 'Filehandle STDIN opened only for output' warnings
-    unless (open(STDIN, ">&SAVEIN")) {
-        $err->trap( error => "couldn't restore STDIN: $!" );
-        return 0;
-    }
-
-    return 1;
-}
-
-
 1;
+
+# Local variables:
+# c-indentation-style: bsd
+# c-basic-offset: 4
+# indent-tabs-mode: nil
+# End:
+# vim: expandtab shiftwidth=4:
